'use strict';

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

exports.Component = Component;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _create = require('./create');

var _create2 = _interopRequireDefault(_create);

var _diff = require('./diff');

var _diff2 = _interopRequireDefault(_diff);

var _patch = require('./patch');

var _patch2 = _interopRequireDefault(_patch);

var _constant = require('./constant');

var _util = require('util');

function Component(props) {
	this.$cache = {
		keepSilent: false
	};
	this.props = props;
	this.state = {};
	this.refs = {};
}

Component.prototype = {
	constructor: Component,
	getDOMNode: function getDOMNode() {
		return this.node;
	},
	setState: function setState(nextState, callback) {
		var $cache = this.$cache;
		var state = this.state;
		var props = this.props;
		var node = this.node;

		if (_util.isFn(nextState)) {
			nextState = nextState(state, props);
		}
		var keepSilent = $cache.keepSilent;

		nextState = _extends({}, this.state, nextState);
		if (keepSilent) {
			$cache.nextState = nextState;
			return;
		}
		var shouldUpdate = this.shouldComponentUpdate(nextState, props);
		this.state = nextState;
		if (shouldUpdate === false) {
			return;
		}
		this.forceUpdate(callback);
	},
	shouldComponentUpdate: function shouldComponentUpdate(nextProps, nextState) {
		return true;
	},
	componentWillUpdate: function componentWillUpdate(nextProps, nextState) {},
	componentDidUpdate: function componentDidUpdate(prevProps, prevState) {},
	componentWillReceiveProps: function componentWillReceiveProps(nextProps) {},
	componentWillMount: function componentWillMount() {},
	componentDidMount: function componentDidMount() {},
	componentWillUnmount: function componentWillUnmount() {},
	forceUpdate: function forceUpdate(callback) {
		var vnode = this.vnode;
		var node = this.node;
		var $cache = this.$cache;
		var state = this.state;
		var props = this.props;
		var id = this.$id;

		if (!node) {
			return;
		}
		var nextProps = $cache.props || props;
		var nextState = $cache.state || state;
		$cache.props = $cache.state = null;
		this.componentWillUpdate(nextProps, nextState);
		this.props = nextProps;
		this.state = nextState;
		_util.setComponentId(id);
		var nextVnode = this.render();
		var patches = _diff2['default'](vnode, nextVnode);
		var newNode = _patch2['default'](node, patches);
		_util.resetComponentId();
		this.refs = _util.getRefs(id);
		// update this.node, if component render new element
		if (newNode !== node) {
			_util.setAttr(newNode, _constant.COMPONENT_ID, id);
			this.node = newNode;
		}
		this.vnode = nextVnode;
		_util.$triggerOnce(_constant.DID_MOUNT);
		this.componentDidUpdate(props, state);
		if (_util.isFn(callback)) {
			callback();
		}
	}
};
var findDOMNode = function findDOMNode(node) {
	return node.nodeName ? node : node.getDOMNode();
};

exports.findDOMNode = findDOMNode;
var combineMixin = function combineMixin(proto, mixin) {
	Object.keys(mixin).forEach(function (key) {
		var source = mixin[key];
		var currentValue = proto[key];
		if (_util.isUndefined(currentValue)) {
			proto[key] = source;
		} else if (_util.isFn(currentValue) && _util.isFn(source)) {
			proto[key] = _util.pipe(currentValue, source);
		}
	});
};
var combineMixins = function combineMixins(proto, mixins) {
	mixins.forEach(function (mixin) {
		return combineMixin(proto, mixin);
	});
};

var bindContext = function bindContext(obj, source) {
	Object.keys(source).forEach(function (key) {
		if (_util.isFn(source[key])) {
			obj[key] = source[key].bind(obj);
		}
	});
};

var createClass = function createClass(options) {
	var mixins = options.mixins || [];
	var defaultProps = _util.isFn(options.getDefaultProps) ? options.getDefaultProps() : null;
	var mixinsForDefaultProps = undefined;
	if (_util.isObj(defaultProps)) {
		mixinsForDefaultProps = {
			componentWillReceiveProps: function componentWillReceiveProps(nextProps) {
				Object.keys(defaultProps).forEach(function (key) {
					if (_util.isUndefined(nextProps[key])) {
						nextProps[key] = defaultProps[key];
					}
				});
			}
		};
		mixins = mixins.concat(mixinsForDefaultProps);
	}
	function Klass(props) {
		Component.call(this, props);
		bindContext(this, Klass.prototype);
		if (_util.isObj(defaultProps)) {
			mixinsForDefaultProps.componentWillReceiveProps(props);
		}
		if (_util.isFn(this.getInitialState)) {
			this.state = this.getInitialState();
		}
	}
	Klass.prototype = Object.create(Component.prototype);
	combineMixins(Klass.prototype, mixins.concat(options));

	if (_util.isObj(options.statics)) {
		Object.keys(options.statics).forEach(function (key) {
			Klass[key] = options.statics[key];
		});
	}
	return Klass;
};

exports.createClass = createClass;
var components = {};
var removeComponent = function removeComponent(id) {
	var component = components[id];
	if (!component) {
		return;
	}
	if (_util.isArr(component)) {
		return component.forEach(function (item) {
			item.componentWillUnmount();
			delete components[item.$id];
		});
	}
	component.componentWillUnmount();
	delete components[id];
};
var checkUnmount = function checkUnmount(node, newNode) {
	if (!node || node.nodeType === 3) {
		return;
	}
	var id = _util.getAttr(node, _constant.COMPONENT_ID);
	// if newNode is existed, it must be calling replaceChild function
	if (id && !newNode) {
		removeComponent(id);
	}
	var componentNodes = _util.querySelectorAll(node, '[' + _constant.COMPONENT_ID + ']');
	_util.toArray(componentNodes).forEach(function (child) {
		return checkUnmount(child);
	});
};

_util.$on(_constant.WILL_UNMOUNT, checkUnmount);

var initComponent = function initComponent(Component, props) {
	props = _extends({}, props, Component.defaultProps);
	var component = new Component(props);
	var id = component.$id = _util.getUid();
	var $cache = component.$cache;

	if (props.ref) {
		_util.collectRef(props.ref, component);
	}
	$cache.keepSilent = true;
	component.componentWillMount();
	$cache.keepSilent = false;
	component.state = $cache.nextState || component.state;
	$cache.nextState = null;
	var vnode = component.vnode = component.render();
	_util.setComponentId(id);
	var node = component.node = _create2['default'](vnode);
	_util.resetComponentId();
	component.refs = _util.getRefs(id);
	var attr = _util.getAttr(node, _constant.COMPONENT_ID);
	if (!attr) {
		_util.setAttr(node, _constant.COMPONENT_ID, attr = id);
	}
	if (components[attr]) {
		if (!_util.isArr(components[attr])) {
			components[attr] = [components[attr]];
		}
		components[attr].splice(0, 0, component);
	} else {
		components[attr] = component;
	}
	_util.$on(_constant.DID_MOUNT, function () {
		$cache.keepSilent = true;
		component.componentDidMount();
		$cache.keepSilent = false;
		if ($cache.nextState) {
			(function () {
				var nextState = $cache.nextState;
				$cache.nextState = null;
				_util.nextFrame(function () {
					component.state = nextState;
					component.forceUpdate();
				});
			})();
		}
	});
	return { component: component, node: node };
};

exports.initComponent = initComponent;
var updateComponent = function updateComponent(component, props) {
	props = _extends({}, props, component.constructor.defaultProps);
	if (props.ref) {
		_util.collectRef(props.ref, component);
	}
	var $cache = component.$cache;

	$cache.keepSilent = true;
	component.componentWillReceiveProps(props);
	$cache.keepSilent = false;
	if ($cache.nextState) {
		component.state = $cache.nextState;
		$cache.nextState = null;
	}
	var shouldUpdate = component.shouldComponentUpdate(props, component.state);
	if (!shouldUpdate) {
		return;
	}
	$cache.props = props;
	$cache.state = component.state;
	component.forceUpdate();
};
exports.updateComponent = updateComponent;