'use strict';

exports.__esModule = true;

var _constant = require('./constant');

var _util = require('util');

/**
* diff vnode and newVnode
*/
var diff = function diff(vnode, newVnode) {
	var children = undefined;
	var type = undefined;
	switch (true) {
		case vnode === newVnode:
			return null;
		case _util.isUndefined(newVnode):
			type = _constant.REMOVE;
			break;
		case _util.isUndefined(vnode):
			type = _constant.CREATE;
			break;
		case vnode === null || newVnode === null || vnode.tagName !== newVnode.tagName:
			type = _constant.REPLACE;
			break;
		case _util.isComponentClass(vnode.tagName):
			type = _constant.UPDATE;
			break;
		case !!(vnode.props || newVnode.props):
			if (_util.hasKey(newVnode) && newVnode.props.key !== vnode.props.key) {
				type = _constant.REPLACE;
			} else {
				type = _constant.PROPS;
			}
			break;
		case !_util.isObj(vnode) && !_util.isObj(newVnode) && vnode != newVnode:
			type = _constant.REPLACE;
			break;
	}
	if (!type || type === _constant.PROPS) {
		if (vnode.props && vnode.props.dangerouslySetInnerHTML || newVnode.props && newVnode.props.dangerouslySetInnerHTML) {} else {
			var childrenType = diffChildren(vnode.children, newVnode.children);
			return { type: type, vnode: vnode, newVnode: newVnode, childrenType: childrenType };
		}
	}
	return type ? { type: type, vnode: vnode, newVnode: newVnode } : null;
};

exports['default'] = diff;

var diffChildren = function diffChildren(children, newChildren) {
	var childrenType = undefined;
	if (!newChildren) {
		childrenType = _constant.REMOVE;
	} else if (!children) {
		childrenType = _constant.CREATE;
	} else {
		childrenType = _constant.REPLACE;
	}
	return childrenType;
};
module.exports = exports['default'];

//pass